/* 
    ORPP Robot Client

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.com
    
    file: display.cpp
*/

#include <stdlib.h>
#include <stdio.h>
#include <windows.h>

#include "display.h"
#include "types.h"
#include "image.h"
#include "robot.h"


extern HWND main_win;                      // Handle to main window
//extern HDC main_dc;                        // Main device context

extern unsigned int fps;

extern unsigned int image_x_size;
extern unsigned int image_y_size;

extern unsigned int sonar;

extern color_type *image_buffer;
extern BITMAPINFO image_buffer_info; 

extern robot_data_type robot_data;

extern char * compressed_image;

int display_x  =  320;                     // Sample ares size in pixels
int display_y  =  200;

static char text_area[text_y_size][text_x_size];  // Data for text display
        

bool display_fps = true;

/* Display the text area */
void display_text( HDC hdc )
{
    // Clear the old text area
    Rectangle(hdc, 0, image_y_size, image_x_size + sonar_bar_size, image_y_size+ text_area_height+2);
        
    // Draw text lines
    for (unsigned int r = 0; r < text_y_size; r++)
        TextOut(hdc, 2, r * font_y_size + image_y_size + 1, text_area[r], strlen(text_area[r]));

  //  for (unsigned int r = 0; r < text_y_size; r++)
//        TextOut(hdc, 2, display_y - r * font_y_size + 1, text_area[r], strlen(text_area[r]));

//        TextOut(hdc, 2, display_y-text_area_height, text_area[0], strlen(text_area[0]));

//    if (main_win != NULL)
  //      InvalidateRect (main_win, NULL, FALSE);
}
/* End if display_text */


/* Called when windows is resized to modify the display area size */
void display_resize(unsigned int new_x, unsigned int new_y)
{
  /*  display_x = new_x; 
    display_y = new_y - text_area_height - 2;
    
    display_text();*/
}
/* End of display_resize */



void draw_sensors( HDC hdc )
{
    // Scale sonar into display
    unsigned int size = image_y_size - (image_y_size  * robot_data.sonar / 16000);

    if (size > image_y_size  - 1) size = image_y_size  -1;

    HBRUSH color1 = CreateSolidBrush(RGB(150, 150, 150));
    HBRUSH color2 = CreateSolidBrush(RGB(0, 0, 255));

    RECT r;

    r.left = image_x_size;
    r.right = image_x_size + sonar_bar_size;

    r.top = 0;
    r.bottom = size;

    FillRect(hdc, &r,color1);

    r.top = size;
    r.bottom = image_y_size;

    FillRect(hdc, &r,color2);
    
    DeleteObject(color1);
    DeleteObject(color2);    
}
      
      



/* Show the text buffer on the screen */
void display(HWND hwnd)
{

PAINTSTRUCT ps;
    
    HDC hdc = BeginPaint(main_win, &ps);//GetDC(main_win);
/*    if (image_buffer) 


        StretchDIBits(hdc, 
                      0, 0, display_x-sonar_bar_size, display_y, 
                      0, 0, image_x_size, image_y_size,               
                      image_buffer, &image_buffer_info, 
                      DIB_RGB_COLORS, SRCCOPY);             */

    if (image_buffer) 
        SetDIBitsToDevice(hdc, 0, 0, 
                          image_x_size, image_y_size,
                          0,0,0,image_y_size, 
                          image_buffer, 
                          &image_buffer_info, DIB_RGB_COLORS);


    draw_sensors(hdc);

      
      
      
    if (display_fps)
    {
        // Display text information
        char b[400];
        sprintf(b, "FPS: %3d", fps);
        TextOut(hdc, 2, 2, b, strlen(b));
    }

    display_text(hdc);

  //  ReleaseDC(main_win, hdc);

EndPaint(main_win, &ps);
    //ValidateRect(main_win, NULL);
}
/* End of display_text_area */


/* Reset the text area */
void clear_text_area( void )
{
    memset(text_area, 0x00, text_y_size * text_x_size);
}
/* End fo clear text area */


/* Adds a line to the text buffer */
void add_line(char *line)
{    
    static unsigned int text_y = 0; 
    
    static bool text_clear = false;
    
    if (!text_clear)
    {
        clear_text_area();
        text_clear = true;
    }    

    // Scrool the display up
    if (text_y == text_y_size)
    {
        for (int i = 1; i < text_y_size; i++)
            memcpy(text_area[i-1], text_area[i], text_x_size);
    }
    else
        text_y ++;
    // Copy the string
    unsigned int line_len = strlen(line);

    strncpy(text_area[text_y - 1], line, text_x_size);

    InvalidateRect (main_win, NULL, FALSE);
//    display_text();
}
/* End of add line */


/* Make a color */
color_type get_color(unsigned char r, unsigned char g, unsigned char b)
{
    color_type t;
    
    t.r = r; t.b = b; t.g = g;
    
    return(t);
}
/* End of get_color */



















