/* 
    ORPP Robot Client

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.com
    
    file: input.cpp
*/

#include <windows.h>
#include <math.h>
#include <stdio.h>

#include "main.h"
#include "input.h"
#include "robot.h"
#include "display.h"
#include "network.h"
#include "timer.h"

bool keys[256] = {0}; // Key table

extern bool running;
extern bool display_fps;

extern int display_x;
extern int display_y;

bool debug = false;


extern bool targeting;
extern bool d_frame;

extern robot_data_type robot_data;

bool update_camera = false;

double last_camera_update = 0;


bool in_motion = false;
double motion_time = 0;



/* handle mouse movement */
void process_mouse( LPARAM lParam )
{
    if (!update_camera) return;

//    unsigned int pos_x = lParam & 0xffff; // decode position
//    unsigned int pos_y = ; 
     
    double current = get_time();

    if (current - last_camera_update > 0.01)
    {
        last_camera_update = current;

        robot_update_pan_tilt(lParam & 0xffff, lParam >> 16);
    }

}
/* end of process_mouse */


/* Set the current robot motion */
void set_motion(unsigned char ls, unsigned char rs, unsigned char ld, unsigned char rd, bool &m)
{
    robot_data.l_speed = ls;
    robot_data.r_speed = rs;
    
    robot_data.l_dir = ld;
    robot_data.r_dir = rd;  
    m = true;
}
/* End of set_motion */



/* Called when a key is pressed */
void key_down(WPARAM wParam,  LPARAM lParam)
{
    if (lParam & REPEAT_CODE) return;        // Ignore repeat keys

    keys[wParam % 0xff] = true;

    bool update = false; // true if robot updated needed
    bool motion_update = false; // true if robot updated needed

    if (keys['K']) { robot_data.l_light = !robot_data.l_light; update = true; }
    if (keys['L']) { robot_data.r_light = !robot_data.r_light; update = true; }
   
    if (wParam == VK_ESCAPE) system_shutdown();     // Escape key

    if (wParam == 'F') display_fps = !display_fps;
    if (wParam == 'X') debug = !debug;
    if (wParam == 'C') update_camera = !update_camera;

    if (wParam == 'T') targeting = !targeting;
    if (wParam == 'R') d_frame = !d_frame;

    if (wParam == 'W') set_motion(255, 255, 0, 0, motion_update); else
    if (wParam == 'S') set_motion(255, 255, 1, 1, motion_update); else
    if (wParam == 'A') set_motion(200, 200, 1, 0, motion_update); else
    if (wParam == 'D') set_motion(200, 200, 0, 1, motion_update); 
 
    if (motion_update)
    {    
        in_motion = true;
        motion_time = get_time();

        robot_update();
    }

    if (update) robot_update();

}
/* End of process key down */


/* Update speeds for accel curves */
void check_input( void )
{
    double current = get_time();        // Get current time
            
    // Moving ?
    if (in_motion)
    {        
         
//        sprintf(b, "Speed: %d", speed);
//        add_line(b);
                  
        if ((robot_data.l_dir == 1 && robot_data.r_dir == 1 ) ||
            (robot_data.l_dir == 0 && robot_data.r_dir == 0 ))
        {
            double down_time = current - motion_time;                 
            if (down_time > 2) down_time = 2;

            unsigned char speed = (unsigned char)(255 * down_time / 2.0);

         
            robot_data.l_speed = speed;
            robot_data.r_speed = speed;  

           
        } else
        {
            double down_time = current - motion_time;                 
            if (down_time > 0.5) down_time = 0.5;
            
            unsigned char speed = (unsigned char)(200 * down_time * 2);
         
            robot_data.l_speed = speed;
            robot_data.r_speed = speed;  

          /*  char b[400];
            sprintf(b, "Speed: %d", speed);
            add_line(b);                                    */
        }

        robot_update();
    }
        
}




/* Called when a key is released */
void key_up( WPARAM wParam )
{
    keys[wParam % 256] = false;
    
    
    if (!keys['W'] && !keys['S'] && !keys['A'] && !keys['D'])
    {
        robot_data.l_speed = 0; 
        robot_data.r_speed = 0; 

      //  robot_data.l_dir = 0; 
       // robot_data.r_dir = 0; 
        
        robot_update();
        in_motion = false;
     }
    
}
/* End of process key up */




