/* 
    ORPP Robot Client

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.com
    
    file: main.cpp
*/

#include <windows.h>
#include <stdio.h>

#include "main.h"
#include "window.h"
#include "display.h"
#include "network.h"
#include "input.h"
#include "image.h"
#include "robot.h"
#include "config.h"
#include "timer.h"


HWND system_hwnd = NULL;                        // Keep track of application handle

unsigned int frames = 0;                        // Numbers of frames so far in this second
unsigned int fps = 0;                           // Frames per second


/* Runs when program startsis created */
void system_startup()
{
    load_config("config.txt");

    robot_start();

    network_start();
 
    SetTimer(system_hwnd, ID_SECOND, 1000, NULL);
    SetTimer(system_hwnd, ID_UPDATE, 10, NULL);  // Update robot 4 times a second
}
/* End of system_startup */


/* Runs when window is closed */
void system_shutdown(void)
{
    robot_reset();

    image_stop();
    
    network_stop();
    
    WSACleanup();

    PostQuitMessage(0);   
}
/* End of system_shutdown */



/* Deal with the system timer */
void timer_message(WPARAM wParam)
{
    if (wParam == ID_SECOND)
    {
        fps = frames;
        frames = 0;
      
        network_check();            // Make sure network is still conneted
               
/*char b[400];
sprintf(b, "Time: %f", get_time());
add_line(b);               */
               
               
               
        return;
    } 
    
    if (wParam == ID_UPDATE)
    {
        request_sensor_data();
        check_input();

        return;
    }    
}   
/* End of timer message */


/* Ehe message handler */
LRESULT CALLBACK WinProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)                 
    {
        case WM_CREATE:     system_hwnd = hwnd;;    break;
        case WM_TIMER:      timer_message(wParam); break;
        case WM_DESTROY:    system_shutdown(); break;
        case WM_KEYDOWN:    key_down(wParam, lParam); break;
        case WM_KEYUP:      key_up(wParam); break;

        case WM_PAINT:      display(hwnd);  break;


        case WM_MOUSEMOVE:  process_mouse(lParam); break;
   
        case WM_SIZE: 
            display_resize(lParam & 0xffff, lParam >> 16); 
            break;
            
        case WM_NETEVENT: network_message(wParam, lParam); break;            
            
        default: return DefWindowProc (hwnd, message, wParam, lParam);
    }

    return(0);
}
/* End of winproc */


/* Winmain */
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    // Make the main window
    if (setup_window(hinstance, nCmdShow)) return(1);

    system_startup();

    MSG messages;

    while (GetMessage (&messages, NULL, 0, 0))
    {
        TranslateMessage(&messages);
        DispatchMessage(&messages);
    }

    return(messages.wParam);
}
/* End of winmain */



