/* 
    ORPP Robot Client

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.com
    
    file: network.h
*/

#include <windows.h>
#include <stdio.h>

#include "main.h"
#include "types.h"
#include "robot.h"
#include "display.h"

#include "network.h"

extern HWND main_win;


extern unsigned int display_x;
extern unsigned int display_y;

extern unsigned int image_x_size;
extern unsigned int image_y_size;

extern bool targeting;

//extern HWND main_win; 
//extern HWND system_hwnd;

robot_data_type robot_data;                 // Actual robot data

void robot_start( void )
{
    robot_clear_values();
}


/* Send all data to robot */
void robot_update( void )
{
    unsigned char  buff[30];
    
    buff[0] = 0x30; // command to update robot data

    buff[1] = robot_data.pan;
    buff[2] = robot_data.tilt;
    buff[3] = robot_data.r_speed;
    buff[4] = robot_data.l_speed;

    buff[5] = 0;
    
    if (robot_data.l_dir) buff[5] |= 0x01;
    if (robot_data.r_dir) buff[5] |= 0x02;

    if (robot_data.l_light) buff[5] |= 0x04;
    if (robot_data.r_light) buff[5] |= 0x08;
        
/*    char b[400];
    sprintf(b, "l: %d  r: %d",  buff[4],  buff[3]);
    add_line(b);*/

    network_send(buff, 6);
}
/* End of robot_data */


/* End of robot clear values */
void robot_clear_values( void )
{
    robot_data.l_speed = 0;
    robot_data.r_speed = 0;    

    robot_data.l_dir = 0;
    robot_data.r_dir = 0;    

    robot_data.l_light = 0;
    robot_data.r_light = 0;    

    robot_data.pan = 0x80;
    robot_data.tilt = 0x80;    

    robot_data.sonar = 0; 
}
/* End of robot clear values */


void robot_reset( void )
{
    robot_clear_values();

    robot_update();     
}


void request_sensor_data( void )
{
    network_send(ROBOT_SENSOR_DATA);
}


/* Deal with sensor data from server */
// Size is the amount of data the server has
// Return the number of bytes used
//
int robot_sensor_data( unsigned char * data, unsigned int size )
{
    if (size < 2) return(0);
    
    robot_data.sonar = (data[0] << 8) + data[1];                              

    //InvalidateRect(main_win, NULL, 0);
    //display();     
    
    return(2); // used two bytes
}
/* End of robot_sensor_data */


/* Update the pan and tilt servo values */
void robot_update_pan_tilt(unsigned int x, unsigned int y)
{
       // range check
  //  if (!targeting) return;

    if (y > display_y - 1) return;


    if (x < 0) x = 0;
    if (x > image_x_size - 1) x = image_x_size - 1;

    if (y < 0) y = 0;
    if (y > image_y_size - 1) y = image_y_size - 1;
        
    const int tilt_upper_lim = 0xff;
    const int tilt_lower_lim = 0x1f;
    
    const int pan_upper_lim = 0xff;
    const int pan_lower_lim = 0x00;  
     
    // scale into servo pos
    robot_data.pan = 255 - pan_lower_lim - (unsigned char)(((pan_upper_lim - pan_lower_lim)/ double(image_x_size)) * x);
    robot_data.tilt = 255 - tilt_lower_lim - (unsigned char)(((tilt_upper_lim - tilt_lower_lim)/ double(image_y_size)) * y);

    robot_update();
}
/* End of robot_update_pan_tilt */

