/* 
    ORPP Robot Server

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.com
    
    file: camera.cpp
*/


#include <windows.h>
#include <stdio.h>

#include "types.h"
#include "window.h"
#include "display.h"
#include "vfwmod.h"
#include "compress.h"
#include "network.h"

extern HWND main_win;                   // Handle for main windows
extern HDC main_dc;                     // Device context for main window

HWND capture_win = NULL;                // Handle to the capture window

unsigned int camera_x_size = 0;         // Size of camera image
unsigned int camera_y_size = 0;

byte *compressed_buffer = NULL;   // Buffer for compressed data
color_type *transcode_buffer = NULL;    // Buffer for transcoding
color_type *camera_buffer = NULL;       // Buffer for camera data

BITMAPINFO camera_buffer_info;          // Bitmap info for camera

HANDLE camera_thread_handle = NULL;
unsigned long camera_thread_id;
bool camera_thread_running = 0;


HANDLE request_image = NULL;


bool running = true;                    // True if we are pulling frames  

bool fast_capture = false;              // True if using direct capture

int camera_number = 0;                  // Camera index

int max_compressed = 0;

/* This is called when the capture is done */
static LRESULT CALLBACK frame_callback(HWND hWnd, LPVIDEOHDR lpVHdr)
{
    if (camera_buffer == NULL) return(true);
 
    if (!fast_capture) return(true);
     
    // Turn stream data into a bitmap
    HBITMAP tmp_bitmap = CreateBitmap(camera_x_size, camera_y_size, 1, 24, lpVHdr->lpData);

    // Transfor into image array
    GetDIBits(main_dc, tmp_bitmap, 0, camera_y_size, camera_buffer, &camera_buffer_info, DIB_RGB_COLORS);
      
    DeleteObject(tmp_bitmap);

    return(true); 
}
/* End of frame_callback */


/* Get a fram from the camera */
void camera_frame(void)
{
    if (!capture_win || !camera_buffer || !running) return;
         
    /*capGrabFrame(capture_win);  // Capture frame and wait for callback

    if (!fast_capture)
    {    
        capEditCopy(capture_win);
			
        OpenClipboard(NULL);
	
        HBITMAP tmp =(HBITMAP) GetClipboardData(CF_BITMAP);
	
        CloseClipboard();
	
        // Transfor into image array
        GetDIBits(main_dc, tmp, 0, camera_y_size, camera_buffer, &camera_buffer_info, DIB_RGB_COLORS);
       	
        DeleteObject(tmp);
    }*/
}
/* End of get frame */



/* Setup buffer for camera image */
void camera_setup_buffer( void )
{    
    if (camera_buffer != NULL) free(camera_buffer);    

    camera_buffer = (color_type *) malloc(camera_x_size * camera_y_size * sizeof(color_type));
        
    if (camera_buffer == NULL)
    {            
        add_line("Unable to create camera buffer");
        return;
    }        

    if (compressed_buffer != NULL) free(compressed_buffer);    

    max_compressed = camera_x_size * camera_y_size * sizeof(color_type) * 2;

    compressed_buffer = (byte *) malloc(max_compressed);
        
    if (compressed_buffer == NULL)
    {            
        add_line("Unable to create compressed buffer");
        return;
    }        



    if (transcode_buffer != NULL) free(transcode_buffer);    

    transcode_buffer = (color_type *) malloc(camera_x_size * camera_y_size * sizeof(color_type));
        
    if (transcode_buffer == NULL)
    {            
        add_line("Unable to create transcode buffer");
        return;
    } 

    // Set the new Bitmap information
    camera_buffer_info.bmiHeader.biSize = sizeof(camera_buffer_info.bmiHeader);
    camera_buffer_info.bmiHeader.biWidth = camera_x_size;
    camera_buffer_info.bmiHeader.biHeight = camera_y_size; 
    camera_buffer_info.bmiHeader.biPlanes = 1;
    camera_buffer_info.bmiHeader.biBitCount = 32;
    camera_buffer_info.bmiHeader.biCompression = BI_RGB;
    camera_buffer_info.bmiHeader.biSizeImage = camera_x_size * camera_y_size  * 4;
    camera_buffer_info.bmiHeader.biClrUsed = 0;
    camera_buffer_info.bmiHeader.biClrImportant = 0;
}
/* End of camera_setup_buffer */


/* Setup the camera buffers and parameters */
void setup_camera( bool ask_info)
{        
    // Get video format information
    if (ask_info)
    {
        capDlgVideoSource(capture_win);
        capDlgVideoFormat(capture_win);         
    }

    // Get video format information
    capGetVideoFormat(capture_win, &camera_buffer_info, sizeof(camera_buffer_info));
   
    if ( camera_buffer_info.bmiHeader.biWidth > 0 &&
         camera_buffer_info.bmiHeader.biHeight > 0)  
    {
        char buff[400];
        sprintf(buff, "Size (%dx%d)", camera_buffer_info.bmiHeader.biWidth,
                                      camera_buffer_info.bmiHeader.biHeight);
                                      add_line(buff);
    }
    else
    {
        add_line("Camera not detected");

    }

    camera_x_size = camera_buffer_info.bmiHeader.biWidth;
    camera_y_size = camera_buffer_info.bmiHeader.biHeight;   

    camera_setup_buffer();
}
/* End of setup camera */



/* Send image to client */
void send_image( void )
{
    unsigned int data_size = 0;

    // Compress the current frame 
    // Offset of 9 makes room for the header
    compress_frame((byte *)camera_buffer, (byte *)(compressed_buffer+9), max_compressed, &data_size, camera_x_size, camera_y_size);        
                               
    // Construct packet header
    compressed_buffer[0] = 0x60; // Server to client command for image block
    
    compressed_buffer[1] = camera_x_size >> 8;       // x size high byte
    compressed_buffer[2] = camera_x_size & 0xff;     // x size low byte

    compressed_buffer[3] = camera_y_size >> 8;       // y size high byte
    compressed_buffer[4] = camera_y_size & 0xff;     // y size low byte
            
    compressed_buffer[5] = (data_size >> 24) & 0xff; // image data 4th byte (high)
    compressed_buffer[6] = (data_size >> 16) & 0xff; // image data 3rd byte 
    compressed_buffer[7] = (data_size >> 8) & 0xff;  // image data 2nd byte 
    compressed_buffer[8] = (data_size) & 0xff;       // image data 1st byte (low)
            
    // Send header and compressed frame data  
    network_send(compressed_buffer, data_size + 9);
}
/* End of send image */


/* Capture frame and wait for client request */
DWORD WINAPI camera_thread( LPVOID lpParam ) 
{
    static unsigned int count = 0;

	while(camera_thread_running)
	{
        if (capture_win && camera_buffer && running)
        {
            capGrabFrame(capture_win);  // Capture frame and wait for callback

            capEditCopy(capture_win);
			
            OpenClipboard(NULL);
	
            HBITMAP tmp =(HBITMAP) GetClipboardData(CF_BITMAP);
	
            CloseClipboard();
	
            // Transfor into image array
            GetDIBits(main_dc, tmp, 0, camera_y_size, camera_buffer, &camera_buffer_info, DIB_RGB_COLORS);
       	
            DeleteObject(tmp);
    
            // Wait for 1/10 of a second for a request, otherwise discard frame
            LRESULT ret = WaitForSingleObject(request_image, 100);
        
          //  ResetEvent(request_image);  // Put request back in unsignaled state

            if (ret == WAIT_OBJECT_0)   // See if the event was singaled
                send_image();           // Send the image to the client
        }
    }

    return(0);
}
/* End of camera thread */


/* Start camera system */
void camera_start( void )
{
    char buff[400];

    capture_win = capCreateCaptureWindow ("Preview", 0, 0, 0, 320, 260,
              HWND_DESKTOP                            , 0); 

//     capture_win = capCreateCaptureWindow ("Preview", WS_VISIBLE, 0, 0, 320, 260,
//              HWND_DESKTOP                            , 0); 
                                       
    if (!capture_win)
    {
        add_line("Unable to create capture window");
        return;
    }

    // Setup Frame callback 
    capSetCallbackOnFrame(capture_win, frame_callback);
               
   // Used for displaying a list of devices 
    char device_name[80];
    char device_version[80];
//    for (int i = 0; i < 10; i++)
        if (capGetDriverDescription (camera_number, device_name, sizeof (device_name), 
                                             device_version, sizeof (device_version)))
        { 
            sprintf(buff, "Camera: %s", device_name);
            add_line(buff);                      
        }

    
//    sprintf(buff, "Using device %d", device_num);
//    add_line(buff);

    capDriverConnect(capture_win, 0);       // connect to driver    

//    capPreviewRate(capture_win, 20);         // Max preview rate
    capPreview(capture_win, FALSE);          // Show preview     

    setup_camera(false);   // setup buffers
    
    
    // Make sure thread start running
	camera_thread_running = true;
	
	request_image = CreateEvent(NULL, 0, 0, NULL);
	
    // Create orpp therad
	camera_thread_handle  = CreateThread(NULL, 0, camera_thread, NULL, 0, &camera_thread_id);      
}
/* End of Start camera */



/* Stop camera system */
void camera_stop( void )
{
    camera_thread_running = false;

    WaitForSingleObject(camera_thread_handle, 5000);

    CloseHandle(request_image);
 
    if (camera_buffer)
        free(camera_buffer);      

    if (compressed_buffer)
        free(compressed_buffer);  
 
    if (!capture_win) return;
 
    SendMessage(capture_win, WM_CAP_DRIVER_DISCONNECT, 0, 0);   
}
/* End of stop_camera */







