/* 
    ORPP Robot Server

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.com
       
    file: main.cpp
*/

#include <windows.h>
#include <stdio.h>

#include "main.h"
#include "window.h"
#include "display.h"
#include "input.h"
#include "camera.h"
#include "network.h"
#include "robot.h"
#include "config.h"


extern char orpp_serial[];                      // Orpp serial port name

extern bool client_connected;                   // True if client is connected

HWND system_hwnd = NULL;                        // Keep track of application handle

unsigned int frames = 0;                        // Numbers of frames so far in this second
unsigned int fps = 0;                           // Frames per second


/* Runs when program starts is created */
void system_startup()
{
    WSADATA wsda;
    WSAStartup(MAKEWORD(1, 1),&wsda);

    load_config("config.txt");

    char b[400];
    sprintf(b, "Using orpp device on  %s", orpp_serial);
    add_line(b);

    camera_start();
        
    startup_network();        
    start_robot();

   // SetTimer(system_hwnd, ID_UPDATE, 10, NULL);     
}
/* End of system_startup */


/* Runs when window is closed */
void system_shutdown(void)
{
    camera_stop();

    stop_robot();

    shutdown_network();
        
    WSACleanup();

    PostQuitMessage(0);   
}
/* End of system_shutdown */



/* Deal with the system timer */
void timer_message(WPARAM wParam)
{
    // Unused
}   
/* End of timer message */


/* Ehe message handler */
LRESULT CALLBACK WinProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)                 
    {
        case WM_CREATE:     system_hwnd = hwnd;     return(0);
        case WM_TIMER:      timer_message(wParam);  return(0);
        case WM_DESTROY:    system_shutdown();      return(0);
        case WM_KEYDOWN:    key_down(wParam);       return(0);
        case WM_KEYUP:      key_up(wParam);         return(0);
        case WM_PAINT:      display();              return(0);
        case WM_NETEVENT:   network_message(wParam, lParam); return(0);

        default: return DefWindowProc (hwnd, message, wParam, lParam);
    }

    return(0);
}
/* End of winproc */


/* Winmain */
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    // Create the main window
    if (setup_window(hinstance, nCmdShow)) return(1);

    system_startup();

    MSG messages;

    while (GetMessage (&messages, NULL, 0, 0))
    {
        TranslateMessage(&messages);
        DispatchMessage(&messages);
    }

    return(messages.wParam);
}
/* End of winmain */



