/* 
    ORPP Robot Server

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.com
   
    file: network.h
*/

#include <windows.h>
#include <stdio.h>

#include "network.h"
#include "main.h"
#include "display.h"
#include "types.h"
#include "camera.h"
#include "compress.h"
#include "robot.h"
#include "orpp.h"

extern unsigned int max_compressed;
extern unsigned char *compressed_buffer;
extern HANDLE request_image;

unsigned int camera_port = 32459;

extern HWND main_win; 
extern HWND system_hwnd;

extern unsigned int frames;

extern color_type *camera_buffer;
extern unsigned int camera_x_size;
extern unsigned int camera_y_size;


SOCKET listen_socket = INVALID_SOCKET;  // Server will listen on this socket
SOCKET client_socket = INVALID_SOCKET;  // This will be the client's socket

bool client_connected = false;

unsigned int client_updates = 0;

/* Start socket system and setup to listen for a client */
void startup_network( void )
{
    // Create listen socket
    listen_socket = socket(AF_INET, SOCK_STREAM, 0);

    if (listen_socket == INVALID_SOCKET)
    {
        add_line("Socket error");
        return;
    }

    u_long arg = 1; // Arg to allow non blocking

    // Make accept non bocking
    if (ioctlsocket(listen_socket, FIONBIO, &arg) != 0)
    {
        add_line("Unable to listen");
        return;
    }

    // Setup parameters
    SOCKADDR_IN server;
    server.sin_addr.s_addr = INADDR_ANY;
    server.sin_port = htons(camera_port);
    server.sin_family = AF_INET;

    // Bind socket 
    if (bind(listen_socket, (LPSOCKADDR) &server, sizeof(struct sockaddr)) == SOCKET_ERROR)
    {
        add_line("Unable to bind");
        return;
    }

    // Defeate Nagle Algorithm 
    BOOL val = TRUE;

    if (setsockopt(listen_socket, IPPROTO_TCP, TCP_NODELAY, (char*)&val, sizeof(BOOL)) == SOCKET_ERROR) 
    {
        add_line("Unable to set TCP_NODELAY");
        return;
    }

    // Setup listen
    if (listen(listen_socket, 1) == SOCKET_ERROR)
    {
        add_line("Unable to listen");
        return;
    }
  
    // Setup message system
    if (WSAAsyncSelect(listen_socket, system_hwnd, WM_NETEVENT, FD_READ | FD_WRITE |
                                                   FD_ACCEPT | FD_CLOSE | FD_CONNECT))
    {
        add_line("WSAAsyncSelect fail");
        return;          
    }    
}
/* End of start network */


/* Stops the network */
void shutdown_network( void )
{
    closesocket(client_socket);
    closesocket(listen_socket);
}
/* End of shutdown_network */   
    

/* Called when a client is trying to connect */
void network_accept(void)
{
    int client_size = sizeof(sockaddr_in);

    // A client is already connected
    if (client_socket != INVALID_SOCKET) 
    {
        sockaddr_in tmp_client; 

        // Accept to a temp socket
        SOCKET tmp_socket = accept(listen_socket,(struct sockaddr*)&tmp_client, &client_size);

        // Close immediately
        closesocket(tmp_socket);

        add_line("Another client attempted to connect");
        return; 
    }

    sockaddr_in client; // Client socket information

    // Accept the client
    client_socket = accept(listen_socket,(struct sockaddr*)&client, &client_size);

    if (client_socket == INVALID_SOCKET)
    {
        add_line("Accept error.");
        return;
    }

    // Display client address
    char out[BUFFER_SIZE];
    sprintf(out, "Connection from: %s", inet_ntoa(client.sin_addr));
    add_line(out);
    
    client_connected = true;
    
    // Prepare first frame for client
    camera_frame();
}
/* End of network accept */



/* Send the currently captured image to the client */
void send_client_image( void )
{
    // Tell the camera thread to send
    SetEvent(request_image);
    
    frames++;
}
/* End of send_client_image */


/* Send Data to client */
int network_send(byte *data, unsigned int size)
{
    // Send the compressed data
    if (send(client_socket, (char *)data, size, 0) == SOCKET_ERROR)     
    {
        return(1);
        client_connected = false;
    } else
        return(0);   
}
/* End of network_send */

            
/* Called when client data is ready */
void network_read(void)
{
    byte buffer[BUFFER_SIZE];

    int n_bytes = recv(client_socket, (char*)buffer, BUFFER_SIZE, 0); // Get the data

    if (n_bytes <= 0) return; // Error
    
    unsigned int i = 0;
    
    while(i < n_bytes)
    {
        if (buffer[i] == ROBOT_MOVEMENT)
        {           
            // Send data to robot module to deal with
            // adjust index based on the amount of data used
            i += robot_movement_data((buffer+i+1), n_bytes - i) + 1;                         
        }else
        if (buffer[i] == ROBOT_SENSOR_DATA) // Sensor data request
        {        
           // robot_send_sensor_data();
            i++;                            // Skip to next                                       
        } else    
        if (buffer[i] == CAMERA_IMAGE)      // Image request from client
        {        
            send_client_image();            // CLient is requesting an image
          //  robot_send_sensor_data();
            i++;                            // Skip to next               
        } else       
            i++;                            // Ignore, skip to attemp resync
    }    
}
/* End of read_client_data */


/* Deal with network messages */
void network_message(WPARAM wParam, LPARAM lParam)
{
    switch (WSAGETSELECTEVENT(lParam))
    {
        case FD_ACCEPT:
            network_accept();
            break;

        case FD_CONNECT:
            break;

        case FD_CLOSE:
            add_line("Client disconnect");
            closesocket(client_socket);
            client_socket = INVALID_SOCKET;
            client_connected = false;            
            break;

        case FD_READ:
            // make sure the socket requesting reading is our client
            if (wParam == client_socket)
                network_read();
            break;
    }
}
/* End of network_message */



