/* 
    ORPP Robot Server

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.com
    
    File robot.cpp
    
*/


#include <windows.h>
#include <stdio.h>

#include "robot.h"
#include "main.h"
#include "orpp.h"
#include "display.h"
#include "network.h"

extern HWND system_hwnd;

unsigned int control_port = 32460;

//SOCKET control_listen_socket = INVALID_SOCKET;      // Socket to listen on
//SOCKET control_client_socket = INVALID_SOCKET;      // CLient connection socket

HANDLE orpp_thread_handle = NULL;
unsigned long orpp_thread_id;
bool orpp_thread_running = 0;

bool orpp_connected = 0;                           // True if connected to board

robot_data_type robot_data, last_robot_data;       // Robot data

char orpp_serial[255] = "COM1";


orpp_class orpp;                   // Instance of orpp device


/* Send all robot values */
void send_robot_values( void )
{
    add_line("Sending value\n");
/*        
char     buff[400];
    
    sprintf(buff, "l: %d  r: %d   ldir:  %d   r:dir  %d", robot_data.l_speed, robot_data.r_speed,
     robot_data.l_dir , robot_data.r_dir );

add_line(buff);
*/
    
    
    
    orpp.set(orpp_servo, 0, robot_data.pan);    
    orpp.set(orpp_servo, 1, robot_data.tilt);

    orpp.set(orpp_digital_out, 0, robot_data.l_light);  
    orpp.set(orpp_digital_out, 1, robot_data.r_light); 

    orpp.set(orpp_pwm, 0, robot_data.r_speed); 
    orpp.set(orpp_pwm, 1, robot_data.l_speed);         

    orpp.set(orpp_opencollector, 1, robot_data.l_dir);        
    orpp.set(orpp_opencollector, 0, robot_data.r_dir); 
}
/* End of send_robot_values */


/* Compare the old and new values, and send the changes */
void check_robot_change(void)
{
    // Has the pan changed?
    if (robot_data.pan != last_robot_data.pan)
    {
        last_robot_data.pan = robot_data.pan;
        orpp.set(orpp_servo, 0, robot_data.pan);
        
char     buff[400];
    
    sprintf(buff, "pan: %d  tilt: %d", robot_data.pan, robot_data.tilt);

add_line(buff);                     
    }
    
    // Has the pan changed?
    if (robot_data.tilt != last_robot_data.tilt)
    {
        last_robot_data.tilt = robot_data.tilt;
        orpp.set(orpp_servo, 1, robot_data.tilt);             
        
        char     buff[400];
    
    sprintf(buff, "pan: %d  tilt: %d", robot_data.pan, robot_data.tilt);

add_line(buff);
    }    

    // Has the left light changed
    if (robot_data.l_light != last_robot_data.l_light)
    {
        last_robot_data.l_light = robot_data.l_light;
        orpp.set(orpp_digital_out, 0, robot_data.l_light);             
    }

    // Has the right light changed
    if (robot_data.r_light != last_robot_data.r_light)
    {
        last_robot_data.r_light = robot_data.r_light;
        orpp.set(orpp_digital_out, 1, robot_data.r_light);             
    }

    // Has the left motor speed changed?
    if (robot_data.l_speed != last_robot_data.l_speed)
    {
        last_robot_data.l_speed = robot_data.l_speed;
        orpp.set(orpp_pwm, 0, robot_data.l_speed);             
    }

    // Has the right motor speed changed?
    if (robot_data.r_speed != last_robot_data.r_speed)
    {
        last_robot_data.r_speed = robot_data.r_speed;
        orpp.set(orpp_pwm, 1, robot_data.r_speed);             
    }

    if (robot_data.l_dir != last_robot_data.l_dir)
    {
        last_robot_data.l_dir = robot_data.l_dir;
        orpp.set(orpp_opencollector, 1, robot_data.l_dir);             
    }

    if (robot_data.r_dir != last_robot_data.r_dir)
    {
        last_robot_data.r_dir = robot_data.r_dir;
        orpp.set(orpp_opencollector, 0, robot_data.r_dir);             
    }





    // toggle led 0 to indicate update
//    orpp.toggle(orpp_led, 0);
}
/* End of update robot */



/* Reset all internal robot data to inital values */
void clear_robot_data( void )
{
    robot_data.l_speed = 0;
    robot_data.r_speed = 0;    

    robot_data.l_dir = 0;
    robot_data.r_dir = 0;    

    robot_data.l_light = 0;
    robot_data.r_light = 0;    

    robot_data.pan = 0x80;          // Servo is centered
    robot_data.tilt = 0x80;    
      
    robot_data.sonar = 0;
   
    last_robot_data = robot_data;   // Copy over to old data
}
/* End of clear_robot_data */
    

/* Send the current sensor data to the client */
void robot_send_sensor_data( void )
{
    unsigned char buff[10];

    // Construct header packet
    buff[0] = 0x90; //Sensor data

    buff[1] = robot_data.sonar >> 8; // sonar high byte
    buff[2] = robot_data.sonar & 0xff; // sonar low byte

//add_line("send sensor");

//    buff[3] = ;
//    buff[4] = ;

//    buff[5] = ;
//    buff[6] = ;

    network_send(buff, 3);
}
/* End of send_sensor_data */ 



/* Deal with movment data from client */
unsigned int robot_movement_data(byte * data, unsigned int size)
{
    if (size < 6) return(0);

    robot_data.pan = data[0];
    robot_data.tilt = data[1];
    robot_data.l_speed = data[2];
    robot_data.r_speed = data[3];

    // the next byte encodes the rest of the data
    unsigned char d = data[4];
          
    robot_data.l_dir = d & 0x01;
    robot_data.r_dir = d & 0x02;

    robot_data.l_light = d & 0x04;
    robot_data.r_light = d & 0x08;
    


    // robot_data.sonar = orpp.get(orpp_sonar, 0);

    //  check_robot_change();

//send_robot_values();

    
    return(5);
}


/* Background thread for interfaceing with serial orpp */
DWORD WINAPI orpp_thread( LPVOID lpParam ) 
{
    static unsigned int count = 0;

	while(orpp_thread_running)
	{
       // Sleep(1);

        count ++;

       // if ((count % 10) == 0)
       //     robot_data.sonar = orpp.get(orpp_sonar, 0);

        check_robot_change();
 
        if (count > 500000)     // Ping and/or attempt to reconnect 10 times per second
        {
            //orpp.ping(); // make sure board is connected

            // Try to reconnect if needed
            if (!orpp.get_comm_ok()) 
            {
              //  add_line("Robot: Not connected to orpp");
                orpp.connect();       
            }

            // Display a change in communication status
            if (orpp_connected != orpp.get_comm_ok()) 
            {
                orpp_connected = orpp.get_comm_ok(); 
               
                if (orpp_connected)
                    add_line("Robot: orpp connected");
                else               
                    add_line("Robot: orpp disconnected");                           
            }
        
            count = 0;
        }

    }

    return(0);
}
/* End of orpp_thread */



/* Start the robot interface subsystem */
void start_robot( void )
{
    // Reset robot values
    clear_robot_data();

    // Setup orpp error and message function
    orpp.set_error_function(add_line);
    orpp.set_message_function(add_line);

    orpp.set_connection(orpp_serial);
    orpp.connect();

    // Make sure thread start running
	orpp_thread_running = true;
	
    // Create orpp therad
	orpp_thread_handle  = CreateThread(NULL, 0, orpp_thread, NULL, 0, &orpp_thread_id);  
}
/* End of start robot */



/* Stop the robot subsystem */
void stop_robot( void )
{
    clear_robot_data();

    Sleep(200); // Make sure the thread updates

	orpp_thread_running = false;
}
/* End of stop robot */



















